/*
Copyright 2019-2020 vChain, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package store

import (
	"github.com/codenotary/immudb/pkg/api/schema"
	"github.com/dgraph-io/badger/v2"
)

func itemToSchema(key []byte, item *badger.Item) (*schema.Item, error) {
	value, err := item.ValueCopy(nil)
	if err != nil {
		return nil, mapError(err)
	}
	if key == nil || len(key) == 0 {
		key = item.KeyCopy(key)
	}
	return &schema.Item{
		Key:   key,
		Value: value,
		Index: item.Version() - 1,
	}, nil
}

func checkKey(key []byte) error {
	if len(key) == 0 || key[0] == tsPrefix {
		return ErrInvalidKey
	}
	return nil
}

func checkSet(key []byte) error {
	if len(key) == 0 || key[0] == tsPrefix {
		return ErrInvalidSet
	}
	return nil
}
